/*
 * (C) Copyright 2011
 * Promwad Innovation Company <www.promwad.com>
 * Written-by: Viktar Palstsiuk <viktar.palstsiuk@promwad.com>
 * 
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 */

#include <common.h>
#include <miiphy.h>
#include <asm/arch/kirkwood.h>
#include <asm/arch/mpp.h>
#include "wtplug.h"

DECLARE_GLOBAL_DATA_PTR;

#ifdef CONFIG_BOARD_IS_PROMWAD_THINCLIENT
#define RED_LED		(1 << 17)
#define GREEN_LED		(1 << 7)
#define POWER_BUTTON_KEY	(1 << 28)
#else
#define POWER_BUTTON_KEY	(1 << 13)
#endif

#ifdef CONFIG_BOARD_IS_PROMWAD_THINCLIENT
void init_leds_and_power_button(void);
void set_led(u32 led, int val);
#endif

int board_early_init_f(void)
{
	/*
	 * default gpio configuration
	 * There are maximum 64 gpios controlled through 2 sets of registers
	 * the  below configuration configures mainly initial LED status
	 */
	kw_config_gpio(WTPLUG_OE_VAL_LOW,
			WTPLUG_OE_VAL_HIGH,
			WTPLUG_OE_LOW, WTPLUG_OE_HIGH);

#ifdef CONFIG_BOARD_IS_PROMWAD_THINCLIENT
	/* Multi-Purpose Pins Functionality configuration */
	u32 kwmpp_config[] = {
		MPP0_NF_IO2,
		MPP1_NF_IO3,
		MPP2_NF_IO4,
		MPP3_NF_IO5,
		MPP4_NF_IO6,
		MPP5_NF_IO7,
		MPP6_SYSRST_OUTn,
		MPP7_GPO,	/* LED_S_G */
		MPP8_TW_SDA,	/* TW_SDA0 */
		MPP9_TW_SCK,	/* TW_SCK0 */
		MPP10_UART0_TXD,
		MPP11_UART0_RXD,
		MPP12_GPO,	
		MPP13_GPIO,	/* PEX0_RST */
		MPP14_GPIO,	/* WLAN_DIS */
		MPP15_GPIO,
		MPP16_GPIO,
		MPP17_GPIO,	/* LED_S_R */
		MPP18_NF_IO0,
		MPP19_NF_IO1,
		MPP20_GE1_0,
		MPP21_GE1_1,
		MPP22_GE1_2,
		MPP23_GE1_3,
		MPP24_GE1_4,
		MPP25_GE1_5,
		MPP26_GE1_6,
		MPP27_GE1_7,
		MPP28_GPIO,	/* BUTTON */
		MPP29_GPIO,	/* LED_WLAN */
		MPP30_GE1_10,
		MPP31_GE1_11,
		MPP32_GE1_12,
		MPP33_GE1_13,
		MPP34_GPIO,
		MPP35_GPIO,	/* PEX0_CLKREQ */
		MPP36_GPIO,	/* TW_SDA1 */
		MPP37_GPIO,	/* TW_SCK1 */
		MPP38_GPIO,	/* GPU_RESET */
		MPP39_AUDIO_I2SBCLK,
		MPP40_AUDIO_I2SDO,
		MPP41_AUDIO_I2SLRC,
		MPP42_AUDIO_I2SMCLK,
		MPP43_AUDIO_I2SDI,		
		MPP44_GPIO,	/* WAKE_WIFI */
		MPP45_GPIO,	/* USB_IND */
		MPP46_GPIO, 	/* USB_RS_IND */
		MPP47_GPIO,	/* RST_GE */
		MPP48_GPIO,	/* RST_USB */
		MPP49_GPIO,	/* PEX0_CLKREQ */
		0
	};	
#else
	u32 kwmpp_config[] = {
		MPP0_NF_IO2,
		MPP1_NF_IO3,
		MPP2_NF_IO4,
		MPP3_NF_IO5,
		MPP4_NF_IO6,
		MPP5_NF_IO7,
		MPP6_SYSRST_OUTn,
		MPP7_GPO,
		MPP8_GPIO,
		MPP9_GPIO,
		MPP10_UART0_TXD,
		MPP11_UART0_RXD,
		MPP12_GPO,
		MPP13_GPIO,
		MPP14_GPIO,
		MPP15_GPIO,
		MPP16_GPIO,
		MPP17_GPIO,
		MPP18_NF_IO0,
		MPP19_NF_IO1,
		MPP20_GE1_0,
		MPP21_GE1_1,
		MPP22_GE1_2,
		MPP23_GE1_3,
		MPP24_GE1_4,
		MPP25_GE1_5,
		MPP26_GE1_6,
		MPP27_GE1_7,
		MPP28_GPIO,
		MPP29_GPIO,
		MPP30_GE1_10,
		MPP31_GE1_11,
		MPP32_GE1_12,
		MPP33_GE1_13,
		MPP34_GPIO,
		MPP35_GPIO,	/* PEX0_CLKREQ */
		MPP36_GPIO,	/* TW_SDA */
		MPP37_GPIO,	/* TW_SCK */
		MPP38_GPIO,	/* LED_S_R */
		MPP39_GPIO,	/* LED_S_G */
		MPP40_GPIO,	/* LED_PLAN */
		MPP41_GPIO,	/* LED_WLAN */
		MPP42_GPIO,	/* PEX0_RST */
		MPP43_GPIO,	/* WLAN_DIS */
		MPP44_GPIO,	/* WAKE_WIFI */
		MPP45_GPIO,	/* USB_IND */
		MPP46_GPIO, 	/* USB_RS_IND */
		MPP47_GPIO,	/* RST_GE */
		MPP48_GPIO,	/* RST_USB */
		MPP49_GPIO,	/* PEX0_CLKREQ */
		0
	};
#endif
	kirkwood_mpp_conf(kwmpp_config);

#ifdef CONFIG_BOARD_IS_PROMWAD_THINCLIENT
	init_leds_and_power_button();
	set_led(RED_LED, 0);
	set_led(GREEN_LED, 1);
#endif

	return 0;
}

int board_init(void)
{
	/*
	 * arch number of board
	 */
	gd->bd->bi_arch_number = MACH_TYPE_WTPLUG;

	/* adress of boot parameters */
	gd->bd->bi_boot_params = kw_sdram_bar(0) + 0x100;

	return 0;
}

#ifdef CONFIG_RESET_PHY_R

#define MV88E1121_PHY_LED_CTRL		16
#define MV88E1121_PHY_LED_PAGE		3
#define MV88E1121_PHY_LED_DEF		0x0030
#define MII_M1121_PHY_SCR		0x10
#define MII_M1121_PHY_SCR_AUTO_CROSS	0x0060

#define RST_GE		(1 << 15)

static inline void mv_phy_88e1121_reset(void)
{
	struct kwgpio_registers *r = (struct kwgpio_registers *)KW_GPIO1_BASE;

	/* Output enable */
	u32 oe = readl(&r->oe);
	writel(oe & ~RST_GE, &r->oe);	/* active low */

	/* Set Low */
	u32 val = readl(&r->dout) & ~RST_GE;
	writel(val, &r->dout);
	
	udelay(20000);	/* T_RESET > 10 ms */
	
	/* Set high */
	val = readl(&r->dout);
	writel(val | RST_GE, &r->dout);
}

void mv_phy_88e1121_init(char *name)
{
	u16 reg;
	u16 devadr;
	
	if (miiphy_set_current_dev(name))
		return;

	/* command to read PHY dev address */
	if (miiphy_read(name, 0xEE, 0xEE, (u16 *) &devadr)) {
		printf("Err..%s could not read PHY dev address\n",
			__FUNCTION__);
		return;
	}

	/*
	 * Enable RGMII delay on Tx and Rx for CPU port
	 * Ref: sec 4.7.2 of chip datasheet
	 */
	miiphy_write(name, devadr, MV88E1121_PGADR_REG, 2);
	miiphy_read(name, devadr, MV88E1121_MAC_CTRL2_REG, &reg);
	reg |= (MV88E1121_RGMII_RXTM_CTRL | MV88E1121_RGMII_TXTM_CTRL);
	miiphy_write(name, devadr, MV88E1121_MAC_CTRL2_REG, reg);
	miiphy_write(name, devadr, MV88E1121_PGADR_REG, 0);
	
	/* reset the phy */
	miiphy_reset(name, devadr);
	
	miiphy_write(name, devadr, MII_M1121_PHY_SCR,
			(MII_M1121_PHY_SCR_AUTO_CROSS | (1<<9) | (1<<8)));
	
	//set leds
	miiphy_write(name, devadr, MV88E1121_PGADR_REG, MV88E1121_PHY_LED_PAGE);
	miiphy_write(name, devadr, MV88E1121_PHY_LED_CTRL, MV88E1121_PHY_LED_DEF);
	miiphy_write(name, devadr, MV88E1121_PGADR_REG, 0);
	
	/* function genphy_config_aneg */
	
	/* Setup standard advertisement */
	miiphy_read(name, devadr, MII_ADVERTISE,(u16 *) &reg);
	reg &= ~(ADVERTISE_ALL | ADVERTISE_100BASE4 | ADVERTISE_PAUSE_CAP |
		 ADVERTISE_PAUSE_ASYM);
	reg |= (ADVERTISE_10HALF | ADVERTISE_10FULL |\
		ADVERTISE_100HALF | ADVERTISE_100FULL |\
		ADVERTISE_PAUSE_CAP | ADVERTISE_PAUSE_ASYM);
	miiphy_write(name, devadr, MII_ADVERTISE, reg);

	/* Configure gigabit if it's supported */
	miiphy_read(name, devadr, MII_CTRL1000, (u16 *) &reg);
	reg |= ADVERTISE_1000FULL;
	miiphy_write(name, devadr, MII_CTRL1000, reg);

	/* restart aneg */
	miiphy_read(name, devadr, MII_BMCR,(u16 *) &reg);
	reg |= (BMCR_ANENABLE | BMCR_ANRESTART);
	/* Don't isolate the PHY if we're negotiating */
	reg &= ~(BMCR_ISOLATE);
	miiphy_write(name, devadr, MII_BMCR, reg);
	
	printf("88E1121 Initialized on %s, devadr %d\n", name, devadr);
}

void reset_phy(void)
{
#if !defined(CONFIG_BOARD_IS_PROMWAD_THINCLIENT)
	mv_phy_88e1121_reset();
#endif
	
	/* configure and initialize both PHY's */
	mv_phy_88e1121_init("egiga0");
	mv_phy_88e1121_init("egiga1");
}
#endif /* CONFIG_RESET_PHY_R */

#ifndef CONFIG_BOARD_IS_PROMWAD_THINCLIENT
#define RED_LED		(1 << 7)
#define GREEN_LED	(1 << 6)
#endif

#define BOTH_LEDS	(GREEN_LED | RED_LED)
#define NEITHER_LED	0

u32 get_power_button_value(void)
{
	struct kwgpio_registers *r;

	r = (struct kwgpio_registers *)KW_GPIO0_BASE;
	return (readl(&r->din) & POWER_BUTTON_KEY);
}

#ifdef CONFIG_BOARD_IS_PROMWAD_THINCLIENT
void set_led(u32 led, int val)
{
	struct kwgpio_registers *r;
	u32 u;

	r = (struct kwgpio_registers *)KW_GPIO0_BASE;
	if (val)
		u = readl(&r->dout) | led;
	else
		u = readl(&r->dout) & ~led;

	writel(u, &r->dout);
}

void init_leds_and_power_button(void)
{
	struct kwgpio_registers *r;

	/*kw_gpio_set_valid(POWER_BUTTON_KEY, GPIO_INPUT_OK);
	kw_gpio_direction_input(POWER_BUTTON_KEY);*/

	r = (struct kwgpio_registers *)KW_GPIO0_BASE;
	writel(readl(&r->oe) | POWER_BUTTON_KEY, &r->oe);
	writel(readl(&r->oe) & ~(RED_LED | GREEN_LED), &r->oe);

	set_led(RED_LED, 0);
	set_led(GREEN_LED, 0);
}

#ifdef CONFIG_NAND_HYNIX
const char TC_DEF_RCVR_BOOTCMD[] = \
	"setenv bootargs ubi.mtd=3 root=ubi0:rootfs rootfstype=ubifs quiet; " \
	"run nand read.e 0x6400000 0x800000 0x400000; " \
	"bootm 0x6400000;";
#else
const char TC_DEF_RCVR_BOOTCMD[] = \
	"setenv bootargs ubi.mtd=3 root=ubi0:rootfs rootfstype=ubifs  quiet; " \
	"run nand read.e 0x6400000 0x500000 0x400000; " \
	"bootm 0x6400000;";
#endif

/*
 * TODO:
 * retrun 1 - rcvr detect, 0 - rcvr not detect, -1 - error
 */
static int tc_recovery_detect(void)
{
	const int RCVR_BTN_HOLD_TIME_MSEC = 3000;

	if (!get_power_button_value())
	{
		ulong start_time;
		ulong elapsed_time;

		start_time = get_timer(0);

		while (1)
		{
			if (get_power_button_value())
			{
				return 0;
			}

			elapsed_time = get_timer(start_time);

			if (elapsed_time > RCVR_BTN_HOLD_TIME_MSEC)
			{
				return 1;
			}
		}
	}

	return 0;
}

int board_late_init(void)
{
	char *tc_is_env_updated;

	tc_is_env_updated = getenv("tc_is_env_updated");
	if ( (tc_is_env_updated == NULL) || (strcmp(tc_is_env_updated, "yes") != 0))
	{
		printf("Update env.\n");

		setenv("tc_is_env_updated","yes");

		/* skip rootfs update */
		setenv("tc_update_rootfs", NULL);

		saveenv();

		do_reset(NULL, 0, 0, NULL);
	}

	if (tc_recovery_detect() == 1)
	{
		char *tc_update_rootfs;
		char *bootcmd_rcvr;

		printf("Start recovery.\n");

		/* сancel the update if it was initiated */
		tc_update_rootfs = getenv("tc_update_rootfs");
		if (tc_update_rootfs != NULL)
		{
			setenv("tc_update_rootfs", NULL);
			saveenv();
		}

		/* init boot in recovery mode */
		bootcmd_rcvr = getenv("bootcmd_rcvr");
		if (bootcmd_rcvr != NULL)
		{
			setenv("bootcmd", "run bootcmd_rcvr");
			setenv("bootdelay", "0");
		}
		else
		{
			setenv("bootcmd", TC_DEF_RCVR_BOOTCMD);
			setenv("bootdelay", "0");
		}
	}

	return 0;
}

static int tc_rootfs_update(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	char *tc_update_rootfs;

	tc_update_rootfs = getenv("tc_update_rootfs");
	if (tc_update_rootfs != NULL)
	{
		if (strcmp(tc_update_rootfs, "yes") == 0)
		{
			char *bootcmd_rcvr;

			printf("Start rootfs update.\n");

			setenv("tc_update_rootfs","started");
			saveenv();

			bootcmd_rcvr = getenv("bootcmd_rcvr");
			if (bootcmd_rcvr != NULL)
			{
				if (run_command(bootcmd_rcvr, 0) == -1)
				{
					if (run_command(TC_DEF_RCVR_BOOTCMD, 0) == -1)
					{
						printf("rootfs update failed\n");
						return 0;
					}
				}
			}
			else
			{
				if (run_command(TC_DEF_RCVR_BOOTCMD, 0) == -1)
				{
					printf("rootfs update failed\n");
					return 0;
				}
			}
		}
		else
		{
			setenv("tc_update_rootfs", NULL);
			setenv("tc_update_type", NULL);
			setenv("tc_update_rootfs_path", NULL);
			setenv("tc_update_user", NULL);
			setenv("tc_update_password", NULL);
			saveenv();
		}
	}

	return 0;
}

U_BOOT_CMD(
	tc_rootfs_update,	1,	1,	tc_rootfs_update,
	"tc_rootfs_update\t- Check whether rootfs need to update and if so updates.\n",
	"Check whether rootfs need to update and if so updates."
);

#else
static void set_leds(u32 leds, u32 blinking)
{
	struct kwgpio_registers *r = (struct kwgpio_registers *)KW_GPIO1_BASE;
	u32 oe = readl(&r->oe) | BOTH_LEDS;
	writel(oe & ~leds, &r->oe);	/* active low */
	u32 bl = readl(&r->blink_en) & ~BOTH_LEDS;
	writel(bl | blinking, &r->blink_en);
}

void show_boot_progress(int val)
{
	switch (val) {
//	case 15:		/* booting Linux */
//		set_leds(GREEN_LED, GREEN_LED);
//		break;
	case 64:		/* Ethernet initialization */
		set_leds(RED_LED, RED_LED);
		break;
	default:
		if (val < 0)	/* error */
			set_leds(BOTH_LEDS, BOTH_LEDS);
		break;
	}
}

int board_late_init(void)
{
  char *s1,*s2;
  struct kwgpio_registers *r;

  r = (struct kwgpio_registers *)KW_GPIO0_BASE;
  writel(readl(&r->oe) | POWER_BUTTON_KEY, &r->oe);

  udelay(10000);
 
  //printf("---BUTTON %d---------\n",get_power_button_value());
  if(!get_power_button_value())
  {
    s1 = getenv("x_bootcmd_kernel_rcvr");
    s2 = getenv("x_bootargs_root_rcvr");
    
    if ((s1 != NULL) &&  (s2 != NULL))
    {
      setenv("x_bootcmd_kernel",s1);
      setenv("x_bootargs_root",s2);
      printf("Boot from recovery image\n");
    }
  }
  
}

#endif

